@@ -1,4 +1,4 @@
-#$Id: clicli.pm 742 2011-01-14 00:14:31Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect/clicli.pm $
+#$Id: clicli.pm 785 2011-05-24 21:08:08Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect/clicli.pm $
 package    #hide from cpan
   Net::DirectConnect::clicli;
 use strict;
@@ -6,14 +6,12 @@ use Net::DirectConnect;
 use Data::Dumper;    #dev only
 $Data::Dumper::Sortkeys = 1;
 no warnings qw(uninitialized);
-our $VERSION = ( split( ' ', '$Revision: 742 $' ) )[1];
+our $VERSION = ( split( ' ', '$Revision: 785 $' ) )[1];
 use base 'Net::DirectConnect';
 
 sub init {
   my $self = shift;
   #$self->log($self, 'inited0',"MT:$self->{'message_type'}", ' with', Dumper  \@_);
-  #%$self = (         #too bad! rewrite
-  #%$self,
   local %_ = (
     #http://www.dcpp.net/wiki/index.php/%24Supports
     'supports_avail' => [ qw(
@@ -170,7 +168,7 @@ sub init {
     'ADCGET' => sub {
       my $self = shift if ref $_[0];
       #$self->log('dev', 'ADCGET', @_);
-      $self->file_send_parse( map { split /\s/, $_ } @_ );
+      $self->cmd( 'Error', "File Not Available" ) if $self->file_send_parse( map { split /\s/, $_ } @_ );
     },
     #};
   );
@@ -233,7 +231,10 @@ sub init {
       my $self = shift if ref $_[0];
       $self->sendcmd( 'ADCSND', @_ );
     },
-    #};
+    'Error' => sub {
+      my $self = shift if ref $_[0];
+      $self->sendcmd( 'Error', $_[0] );
+    },
   );
   $self->{'cmd'}{$_} ||= $_{$_} for keys %_;
 }
