@@ -1,11 +1,11 @@
-#$Id: hub.pm 505 2009-11-22 03:52:21Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect/hub.pm $
+#$Id: hub.pm 593 2010-01-30 11:11:27Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect/hub.pm $
 package    #hide from cpan
   Net::DirectConnect::hub;
 use Net::DirectConnect;
 use Net::DirectConnect::hubcli;
 use strict;
 no warnings qw(uninitialized);
-our $VERSION = ( split( ' ', '$Revision: 505 $' ) )[1];
+our $VERSION = ( split( ' ', '$Revision: 593 $' ) )[1];
 use base 'Net::DirectConnect';
 
 sub init {
@@ -22,7 +22,7 @@ sub init {
     'HubName'       => 'Net::DirectConnect test hub',
     , @_
   );
-  $self->baseinit();
+  #$self->baseinit();
   $self->{'parse'} ||= {};
   $self->{'cmd'}   ||= {};
 }
